# Generated by Django 2.2.20 on 2021-09-20 21:29

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0069_auto_20210909_2140"),
    ]

    operations = [
        migrations.AddField(
            model_name="workflow",
            name="time_general_hours",
            field=models.PositiveIntegerField(default=0, null=True),
        ),
        migrations.AddField(
            model_name="workflow",
            name="time_specific_hours",
            field=models.PositiveIntegerField(default=0, null=True),
        ),
        migrations.AlterField(
            model_name="column",
            name="comments",
            field=models.ManyToManyField(
                blank=True, related_name="column", to="course_flow.Comment"
            ),
        ),
        migrations.AlterField(
            model_name="node",
            name="comments",
            field=models.ManyToManyField(
                blank=True, related_name="node", to="course_flow.Comment"
            ),
        ),
        migrations.AlterField(
            model_name="outcome",
            name="comments",
            field=models.ManyToManyField(
                blank=True, related_name="outcome", to="course_flow.Comment"
            ),
        ),
        migrations.AlterField(
            model_name="week",
            name="comments",
            field=models.ManyToManyField(
                blank=True, related_name="week", to="course_flow.Comment"
            ),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="ponderation_individual",
            field=models.PositiveIntegerField(default=0, null=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="ponderation_practical",
            field=models.PositiveIntegerField(default=0, null=True),
        ),
        migrations.AlterField(
            model_name="workflow",
            name="ponderation_theory",
            field=models.PositiveIntegerField(default=0, null=True),
        ),
    ]
