{% extends "course_flow/base.html" %}
<!-- -->
{% load static i18n %}
<!-- -->
{% block title %}{% trans "Profile" %} - CourseFlow{% endblock %}
<!-- -->
{% block metadescription %} {% endblock %}
<!-- -->
{% block header %}{% endblock %}
<!-- -->
{% block body %}
<div id="create">
    <div class="workflow-create-wrapper">
    <h1>
      {% trans 'Edit User' %}
    </h1>
    <form method="post">
      {% csrf_token %} {{ form.as_p }}
      <input id="save-button" type="submit" value="Save" disabled="true"/>
    </form>
</div>
{% endblock %}
<!-- -->
{% block foot %} {% endblock %}
<!-- -->
{% csrf_token %}
<!-- -->
{% block scripts %} {{ block.super }}
<script nonce="{{request.csp_nonce}}">
  $("#create").detach().appendTo("#container");
  $("form input").on("input",()=>$("#save-button").attr("disabled",false));
  $("#mark-all-as-read").on("click",()=>{
    $.post("{% url 'course_flow:mark-all-as-read' %}",{},()=>{
      window.location = window.location;
    });
  });
</script>
{% endblock %}
