# Generated by Django 3.2.15 on 2023-01-18 21:09

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ("course_flow", "0098_auto_20230118_1413"),
    ]

    operations = [
        migrations.AlterField(
            model_name="courseflowuser",
            name="user",
            field=models.OneToOneField(
                on_delete=django.db.models.deletion.CASCADE,
                primary_key=True,
                related_name="courseflow_user",
                serialize=False,
                to=settings.AUTH_USER_MODEL,
            ),
        ),
    ]
