# Generated by Django 2.2.20 on 2021-08-27 20:42

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0065_auto_20210818_2256"),
    ]

    operations = [
        migrations.CreateModel(
            name="CustomTerm",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                ("term", models.CharField(max_length=50)),
                ("translation", models.CharField(max_length=50)),
            ],
        ),
        migrations.AddField(
            model_name="project",
            name="terminology_dict",
            field=models.ManyToManyField(
                blank=True, to="course_flow.CustomTerm"
            ),
        ),
    ]
