# Generated by Django 3.2.14 on 2022-09-30 15:27

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0088_liveproject"),
    ]

    operations = [
        migrations.AddField(
            model_name="liveproject",
            name="default_all_workflows_visible",
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name="liveproject",
            name="visible_workflows",
            field=models.ManyToManyField(
                blank=True, to="course_flow.Workflow"
            ),
        ),
        migrations.AlterField(
            model_name="objectpermission",
            name="permission_type",
            field=models.PositiveIntegerField(
                choices=[
                    (0, "None"),
                    (1, "View"),
                    (2, "Edit"),
                    (3, "Comment"),
                    (4, "Student"),
                ],
                default=0,
            ),
        ),
    ]
