# Generated by Django 3.2.15 on 2022-11-14 19:09

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("course_flow", "0094_liveassignment_userassignment"),
    ]

    operations = [
        migrations.CreateModel(
            name="UpdateNotification",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "title",
                    models.CharField(blank=True, max_length=200, null=True),
                ),
                (
                    "created_on",
                    models.DateTimeField(default=django.utils.timezone.now),
                ),
            ],
        ),
    ]
